@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2014-2017 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2014-2018 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014-2015 Brett Cannon <brett@python.org>
 # Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
 # Copyright (c) 2015 Cosmin Poieana <cmin@ropython.org>
@@ -8,6 +8,7 @@
 # Copyright (c) 2016 Roy Williams <rwilliams@lyft.com>
 # Copyright (c) 2016 Derek Gustafson <degustaf@gmail.com>
 # Copyright (c) 2017 Daniel Miller <millerdev@gmail.com>
+# Copyright (c) 2018 Sushobhit <31987769+sushobhit27@users.noreply.github.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
@@ -180,18 +181,33 @@ class TestPython3Checker(testutils.CheckerTestCase):
                      'enumerate', 'dict'):
             self.as_argument_to_callable_constructor_test(fxn, func)
 
-    @python2_only
+    def test_dict_subclasses_methods_in_iterating_context(self):
+        iterating, not_iterating = astroid.extract_node('''
+        from __future__ import absolute_import
+        from collections import defaultdict
+        d = defaultdict(list)
+        a, b = d.keys() #@
+        x = d.keys() #@
+        ''')
+
+        with self.assertNoMessages():
+            self.checker.visit_call(iterating.value)
+
+        message = testutils.Message('dict-keys-not-iterating', node=not_iterating.value)
+        with self.assertAddsMessages(message):
+            self.checker.visit_call(not_iterating.value)
+
     def test_dict_methods_in_iterating_context(self):
         iterating_code = [
             'for x in {}: pass',
             '(x for x in {})',
             '[x for x in {}]',
             'func({})',
-            'a, b = {}'
+            'a, b = {}',
         ]
         non_iterating_code = [
             'x = __({}())',
-            '__({}())[0]'
+            '__({}())[0]',
         ]
 
         for method in ('keys', 'items', 'values'):
@@ -212,19 +228,15 @@ class TestPython3Checker(testutils.CheckerTestCase):
                 with self.assertAddsMessages(message):
                     self.checker.visit_call(node)
 
-    @python2_only
     def test_map_in_iterating_context(self):
         self.iterating_context_tests('map')
 
-    @python2_only
     def test_zip_in_iterating_context(self):
         self.iterating_context_tests('zip')
 
-    @python2_only
     def test_range_in_iterating_context(self):
         self.iterating_context_tests('range')
 
-    @python2_only
     def test_filter_in_iterating_context(self):
         self.iterating_context_tests('filter')
 
@@ -384,10 +396,18 @@ class TestPython3Checker(testutils.CheckerTestCase):
                 self.checker.visit_call(node)
 
     def test_dict_iter_method_on_dict(self):
-        node = astroid.extract_node('{}.iterkeys()')
-        message = testutils.Message('dict-iter-method', node=node)
-        with self.assertAddsMessages(message):
-            self.checker.visit_call(node)
+        nodes = astroid.extract_node('''
+        from collections import defaultdict
+        {}.iterkeys() #@
+        defaultdict(list).iterkeys() #@
+        class Someclass(dict):
+            pass
+        Someclass().iterkeys() #@
+        ''')
+        for node in nodes:
+            message = testutils.Message('dict-iter-method', node=node)
+            with self.assertAddsMessages(message):
+                self.checker.visit_call(node)
 
     def test_dict_not_iter_method(self):
         arg_node = astroid.extract_node('x.iterkeys(x)  #@')
@@ -406,10 +426,18 @@ class TestPython3Checker(testutils.CheckerTestCase):
                 self.checker.visit_call(node)
 
     def test_dict_view_method_on_dict(self):
-        node = astroid.extract_node('{}.viewkeys()')
-        message = testutils.Message('dict-view-method', node=node)
-        with self.assertAddsMessages(message):
-            self.checker.visit_call(node)
+        nodes = astroid.extract_node('''
+        from collections import defaultdict
+        {}.viewkeys() #@
+        defaultdict(list).viewkeys() #@
+        class Someclass(dict):
+            pass
+        Someclass().viewkeys() #@
+        ''')
+        for node in nodes:
+            message = testutils.Message('dict-view-method', node=node)
+            with self.assertAddsMessages(message):
+                self.checker.visit_call(node)
 
     def test_dict_not_view_method(self):
         arg_node = astroid.extract_node('x.viewkeys(x)  #@')
@@ -461,7 +489,20 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(message):
             self.checker.visit_raise(node)
 
-    @python2_only
+        node = astroid.extract_node('raise Exception, "test", tb')
+        message = testutils.Message('old-raise-syntax', node=node)
+
+        with self.assertAddsMessages(message):
+            self.checker.visit_raise(node)
+
+    def test_xreadlines_attribute(self):
+        node = astroid.extract_node("""
+        f.xreadlines #@
+        """)
+        message = testutils.Message('xreadlines-attribute', node=node)
+        with self.assertAddsMessages(message):
+            self.checker.visit_attribute(node)
+
     def test_exception_message_attribute(self):
         node = astroid.extract_node("""
         try:
@@ -473,7 +514,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(message):
             self.checker.visit_attribute(node)
 
-    @python2_only
     def test_normal_message_attribute(self):
         node = astroid.extract_node("""
         e.message #@
@@ -481,33 +521,28 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertNoMessages():
             self.checker.visit_attribute(node)
 
-    @python2_only
     def test_invalid_codec(self):
         node = astroid.extract_node('foobar.encode("hex") #@')
         message = testutils.Message('invalid-str-codec', node=node)
         with self.assertAddsMessages(message):
             self.checker.visit_call(node)
 
-    @python2_only
     def test_valid_codec(self):
         node = astroid.extract_node('foobar.encode("ascii", "ignore")  #@')
         with self.assertNoMessages():
             self.checker.visit_call(node)
 
-    @python2_only
     def test_visit_call_with_kwarg(self):
         node = astroid.extract_node('foobar.raz(encoding="hex")  #@')
         with self.assertNoMessages():
             self.checker.visit_call(node)
 
-    @python2_only
     def test_invalid_open_codec(self):
         node = astroid.extract_node('open(foobar, encoding="hex") #@')
         message = testutils.Message('invalid-str-codec', node=node)
         with self.assertAddsMessages(message):
             self.checker.visit_call(node)
 
-    @python2_only
     def test_valid_open_codec(self):
         node = astroid.extract_node('open(foobar, encoding="palmos") #@')
         with self.assertNoMessages():
@@ -544,7 +579,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
             with self.assertAddsMessages(message):
                 self.checker.visit_call(node)
 
-    @python2_only
     def test_sys_maxint(self):
         node = astroid.extract_node('''
         import sys
@@ -554,7 +588,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(message):
             self.checker.visit_attribute(node)
 
-    @python2_only
     def test_itertools_izip(self):
         node = astroid.extract_node('''
         from itertools import izip #@
@@ -564,8 +597,7 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(absolute_import_message, message):
             self.checker.visit_importfrom(node)
 
-    @python2_only
-    def test_deprecated_types_fiels(self):
+    def test_deprecated_types_fields(self):
         node = astroid.extract_node('''
         from types import StringType #@
         ''')
@@ -574,7 +606,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(absolute_import_message, message):
             self.checker.visit_importfrom(node)
 
-    @python2_only
     def test_sys_maxint_imort_from(self):
         node = astroid.extract_node('''
         from sys import maxint #@
@@ -584,7 +615,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(absolute_import_message, message):
             self.checker.visit_importfrom(node)
 
-    @python2_only
     def test_object_maxint(self):
         node = astroid.extract_node('''
         sys = object()
@@ -593,7 +623,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertNoMessages():
             self.checker.visit_attribute(node)
 
-    @python2_only
     def test_bad_import(self):
         node = astroid.extract_node('''
         import urllib2, sys #@
@@ -616,7 +645,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
                 self.checker.visit_importfrom(node)
             self.checker._future_absolute_import = False
 
-    @python2_only
     def test_bad_import_conditional(self):
         node = astroid.extract_node('''
         import six
@@ -627,7 +655,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(absolute_import_message):
             self.checker.visit_import(node)
 
-    @python2_only
     def test_bad_import_try_except_handler(self):
         node = astroid.extract_node('''
         try:
@@ -639,7 +666,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(absolute_import_message):
             self.checker.visit_import(node)
 
-    @python2_only
     def test_bad_import_try(self):
         node = astroid.extract_node('''
         try:
@@ -653,7 +679,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(absolute_import_message):
             self.checker.visit_import(node)
 
-    @python2_only
     def test_bad_import_try_finally(self):
         node = astroid.extract_node('''
         try:
@@ -666,7 +691,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(absolute_import_message, message):
             self.checker.visit_import(node)
 
-    @python2_only
     def test_bad_import_from(self):
         node = astroid.extract_node('''
         from cStringIO import StringIO #@
@@ -676,7 +700,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(absolute_import_message, message):
             self.checker.visit_importfrom(node)
 
-    @python2_only
     def test_bad_string_attribute(self):
         node = astroid.extract_node('''
         import string
@@ -686,7 +709,73 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(message):
             self.checker.visit_attribute(node)
 
-    @python2_only
+    def test_bad_operator_attribute(self):
+        node = astroid.extract_node('''
+        import operator
+        operator.div #@
+        ''')
+        message = testutils.Message('deprecated-operator-function', node=node)
+        with self.assertAddsMessages(message):
+            self.checker.visit_attribute(node)
+
+    def test_comprehension_escape(self):
+        list_comp, set_comp, dict_comp = astroid.extract_node('''
+        [i for i in range(10)]
+        i #@
+        {c for c in range(10)}
+        c #@
+        {j:j for j in range(10)}
+        j #@
+        ''')
+        message = testutils.Message('comprehension-escape', node=list_comp)
+        with self.assertAddsMessages(message):
+            self.checker.visit_name(list_comp)
+
+        for node in (set_comp, dict_comp):
+            with self.assertNoMessages():
+                self.checker.visit_name(node)
+
+    def test_exception_escape(self):
+        bad, good = astroid.extract_node('''
+        try: 1/0
+        except ValueError as exc:
+            pass
+        exc #@
+        try:
+           2/0
+        except (ValueError, TypeError) as exc:
+           exc = 2
+        exc #@
+        ''')
+        message = testutils.Message('exception-escape', node=bad)
+        with self.assertAddsMessages(message):
+            self.checker.visit_name(bad)
+        with self.assertNoMessages():
+            self.checker.visit_name(good)
+
+    def test_bad_sys_attribute(self):
+        node = astroid.extract_node('''
+        import sys
+        sys.exc_clear #@
+        ''')
+        message = testutils.Message('deprecated-sys-function', node=node)
+        with self.assertAddsMessages(message):
+            self.checker.visit_attribute(node)
+
+    def test_bad_urllib_attribute(self):
+        nodes = astroid.extract_node('''
+        import urllib
+        urllib.addbase #@
+        urllib.splithost #@
+        urllib.urlretrieve #@
+        urllib.urlopen #@
+        urllib.urlencode #@
+        ''')
+        for node in nodes:
+            message = testutils.Message('deprecated-urllib-function', node=node)
+            with self.assertAddsMessages(message):
+                self.checker.visit_attribute(node)
+
     def test_ok_string_attribute(self):
         node = astroid.extract_node('''
         import string
@@ -695,7 +784,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertNoMessages():
             self.checker.visit_attribute(node)
 
-    @python2_only
     def test_bad_string_call(self):
         node = astroid.extract_node('''
         import string
@@ -705,7 +793,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(message):
             self.checker.visit_call(node)
 
-    @python2_only
     def test_ok_shadowed_call(self):
         node = astroid.extract_node('''
         import six.moves.configparser
@@ -714,7 +801,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertNoMessages():
             self.checker.visit_call(node)
 
-    @python2_only
     def test_ok_string_call(self):
         node = astroid.extract_node('''
         import string
@@ -723,7 +809,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertNoMessages():
             self.checker.visit_call(node)
 
-    @python2_only
     def test_bad_string_import_from(self):
         node = astroid.extract_node('''
          from string import atoi #@
@@ -733,7 +818,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(absolute_import_message, message):
             self.checker.visit_importfrom(node)
 
-    @python2_only
     def test_ok_string_import_from(self):
         node = astroid.extract_node('''
          from string import digits #@
@@ -742,7 +826,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(absolute_import_message):
             self.checker.visit_importfrom(node)
 
-    @python2_only
     def test_bad_str_translate_call_string_literal(self):
         node = astroid.extract_node('''
          foobar.translate(None, 'abc123') #@
@@ -752,7 +835,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(message):
             self.checker.visit_call(node)
 
-    @python2_only
     def test_bad_str_translate_call_variable(self):
         node = astroid.extract_node('''
          def raz(foobar):
@@ -763,7 +845,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(message):
             self.checker.visit_call(node)
 
-    @python2_only
     def test_bad_str_translate_call_infer_str(self):
         node = astroid.extract_node('''
          foobar = "hello world"
@@ -774,7 +855,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(message):
             self.checker.visit_call(node)
 
-    @python2_only
     def test_ok_str_translate_call_integer(self):
         node = astroid.extract_node('''
          foobar.translate(None, 33) #@
@@ -782,7 +862,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertNoMessages():
             self.checker.visit_call(node)
 
-    @python2_only
     def test_ok_str_translate_call_keyword(self):
         node = astroid.extract_node('''
          foobar.translate(None, 'foobar', raz=33) #@
@@ -790,7 +869,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertNoMessages():
             self.checker.visit_call(node)
 
-    @python2_only
     def test_ok_str_translate_call_not_str(self):
         node = astroid.extract_node('''
          foobar = {}
@@ -825,7 +903,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertNoMessages():
             self.walk(module)
 
-    @python2_only
     def test_versioninfo_conditional(self):
         code = '''
         from __future__ import absolute_import
@@ -838,7 +915,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertNoMessages():
             self.walk(module)
 
-    @python2_only
     def test_versioninfo_tuple_conditional(self):
         code = '''
         from __future__ import absolute_import
@@ -851,7 +927,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertNoMessages():
             self.walk(module)
 
-    @python2_only
     def test_six_ifexp_conditional(self):
         code = '''
         from __future__ import absolute_import
@@ -863,7 +938,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertNoMessages():
             self.walk(module)
 
-    @python2_only
     def test_next_defined(self):
         node = astroid.extract_node("""
             class Foo(object):
@@ -873,7 +947,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(message):
             self.checker.visit_functiondef(node)
 
-    @python2_only
     def test_next_defined_too_many_args(self):
         node = astroid.extract_node("""
             class Foo(object):
@@ -882,7 +955,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertNoMessages():
             self.checker.visit_functiondef(node)
 
-    @python2_only
     def test_next_defined_static_method_too_many_args(self):
         node = astroid.extract_node("""
             class Foo(object):
@@ -892,7 +964,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertNoMessages():
             self.checker.visit_functiondef(node)
 
-    @python2_only
     def test_next_defined_static_method(self):
         node = astroid.extract_node("""
             class Foo(object):
@@ -903,7 +974,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(message):
             self.checker.visit_functiondef(node)
 
-    @python2_only
     def test_next_defined_class_method(self):
         node = astroid.extract_node("""
             class Foo(object):
@@ -933,6 +1003,9 @@ class TestPython3TokenChecker(testutils.CheckerTestCase):
     def test_old_ne_operator(self):
         self._test_token_message("1 <> 2", "old-ne-operator")
 
+    def test_invalid_string_literal(self):
+        self._test_token_message("a = ur'aaa'", 'invalid-unicode-literal')
+
     def test_old_octal_literal(self):
         for octal in ("045", "055", "075", "077", "076543"):
             self._test_token_message(octal, "old-octal-literal")
